#ifndef _curl_included
#define _curl_included

// standard includes 
#include <list>
#include <map>
#include <set>
#include <vector>

#include <pthread.h> // for pthread_mutex_t

// libcurl
#include <curl/curl.h> // for CURL type
#include <event.h>    // for libevent
#include <pthread.h>

// DataExchange types (URI)
#include "DataExchange_types.h"

// Utility class to allow for thread-safe libcurl invokations
class MyMultiCURL 
{
public:
  DataExchangeStatus batchGetData(int n_uris, URI *uris, char **buffers);
  DataExchangeStatus new_conn(const char *url, char*& buffer, int current_uri_pos, CURL *easy=NULL);
  
private:
  typedef struct {
    int no_alloc;
    char *buffer;
    int status;
    char error[CURL_ERROR_SIZE];
    int  cup;
    CURL *easy;
  } CURLUserData;

  typedef struct {
    curl_socket_t sockfd;
    long timeout;
    struct event *ev;
    int evset;
  }CURLSocketData;

  typedef struct {
    CURLM *multicurl;
    URI *uris;
    char **buffers;
    struct event *timer_event;   //struct event timer_event;
    int still_running; 
    int endstatus;
    struct event_base *eventbase;
    MyMultiCURL* handle;
    int n_uris;
    int next_uri_toprocess;
  } CURLGlobalData;

  
  static size_t writeData(void *buffer, size_t size, size_t n, void *user_data);
  static int multicurl_socket_cb(CURL *easy, curl_socket_t s, int action, void *userp, void *socketp); 
  static int multicurl_timer_cb(CURLM *multi, long timeout_ms, void *userp);
  static void check_multi_info(CURLGlobalData *curlglobaldata);
 
  static void libevent_event_cb(int fd, short kind, void *userp);
  static void libevent_timer_cb(int fd, short kind, void *userp)
  static void setsock(CURLSocketData *sockdata, int action, curl_socket_t s, CURLGlobalData *curlglobaldata);
 
  MyMultiCURL(CURLM *c = NULL, bool used = false);
  virtual ~MyMultiCURL(void);  
  
  static pthread_mutex_t mutex;
  static size_t size;
  CURLGlobalData curlglobaldata;

  CURLM *multicurl;
  struct event_base *eventbase;
};

#endif
